'''
 *category   Micropython application
 *package    speak weather forecast main program
 *author     Ichiro Shirasaka
 *copyright  2018 Ichiro Shirasaka All Rights Reserved
 *version    1.0'''

#固定音声の発声関数（/sd/sound/下のmp3ファイルの内容を発声）
def speak(filename):
    vs.VS1003_SCI_Write(0x00, 0x0804)  #音声出力モードに設定
    time.sleep_ms(100)
    vs.VS1003_Volume(240, 240)
    buf = bytearray(512)
    fb = open('/sd/sound/'+filename, "rb")
    data_len = 512
    vs.VS1003_SDI_Enable()
    while data_len == 512:
        data_len = fb.readinto(buf)
        xdcs.value(0)
        for n in range(0, 512, 32):
            vs.VS1003_Wait_dreq()
            spi.write(buf[n:n+32])
        xdcs.value(1)
    fb.close()

#Beep音関数(100ms)
def beep():
    pwm12.duty(512)
    time.sleep_ms(100)
    pwm12.duty(0)

#WiFi アクセスポイント接続関数
def do_connect():
    import network
    sta_if = network.WLAN(network.STA_IF)
    if not sta_if.isconnected():
        print('connecting to network...')
        sta_if.active(True)
        sta_if.connect(' INPUT SSID','INPUT PASSWORD')
        while not sta_if.isconnected():
            pass
    print('network config:', sta_if.ifconfig())

#SDカードドライバ初期化関数
def sddrv():
    import machine, sdcard, os
    sdspi = SPI(-1, sck=Pin(18), mosi=Pin(23), miso=Pin(19))
    sd = sdcard.SDCard(sdspi, machine.Pin(5))
    os.mount(sd,'/sd')

#
#おしゃべり天気予報Main処理
#
import ubinascii
import usocket
import json
from machine import Pin, SPI, PWM
import time, gc
import weather_id
from isound import *

btn = Pin(0, Pin.IN)
pwm12 = PWM(Pin(12))    #Beep用　PWM設定
pwm12.duty(0)           #PWM duty = 0
pwm12.freq(4000)        #Beep周波数4KHz

#音声ボード制御ピン/SPIピン設定
spi  = SPI(1, baudrate=100000, sck=Pin(17), mosi=Pin(26), miso=Pin(25))
xrst = Pin(15, Pin.OUT)
xdcs = Pin(13, Pin.OUT)
xcs  = Pin(14, Pin.OUT)
dreq = Pin(16, Pin.IN)

vs = Vs1003(spi, xrst, xdcs, xcs, dreq) #音声ボードドライバ　インスタンス生成
req = Request(spi, xdcs, dreq)          #HTTPリクエストユーテリティ　インスタンス生成

datalen = 15360                         #録音バッファ（フラッシュメモリ上）サイズ（バイト）
soundheader = bytearray(60)
buf = bytearray(1024)                   #音声出力バッファ
vs.VS1003_Reset()                       #音声ボードハードリセット

#WAV（ADPCM）ファイルヘッダ
soundheader[0:4] = b"RIFF"
soundheader[4:8] = int(datalen+52).to_bytes(4,"little")
soundheader[8:16] = b"WAVEfmt "
soundheader[16:20] = b'\x14\x00\x00\x00'
soundheader[20:24] = b'\x11\x00\x01\x00'
soundheader[24:28] = b'\x40\x1f'
soundheader[28:32] = b'\xd7\x0f'
soundheader[32:34] = b'\x00\x01'
soundheader[34:36] = b'\x04\x00'
soundheader[36:38] = b'\x02\x00'
soundheader[38:40] = b'\xf9\x01'
soundheader[40:44] = b"fact"
soundheader[44:48] = b'\x04\x00\x00\x00'
soundheader[48:52] = int(datalen//256*505).to_bytes(4,"little")
soundheader[52:56] = b"data"
soundheader[56:60] = int(datalen).to_bytes(4,"little")

spi.init(baudrate = 4000000)    #音声ボードSPI Clock=４MHz
do_connect()                    #WiFiへ接続
sddrv()                         #SDカードドライバ初期化
speak("message_connect.mp3")    #開始アナウンス

#天気予報URL
weather_url = 'http://weather.livedoor.com/forecast/webservice/json/v1?city='

#Speech to Text URL
recog_url = 'https://stream.watsonplatform.net/speech-to-text/api/v1/recognize?model=ja-JP_NarrowbandModel'
recog_username = 'Watson user name'
recog_password = 'Watson pass word'
recog_auth = ubinascii.b2a_base64(recog_username +':'+ recog_password)[:-1].decode('utf-8')
recog_headers={'Content-Type': 'audio/wav','Authorization': 'Basic %s' % recog_auth}

#Text to Speech URL
speak_url = 'https://stream.watsonplatform.net/text-to-speech/api/v1/synthesize?voice=ja-JP_EmiVoice&accept=audio/mp3'
speak_username = 'Watson user name'
speak_password = 'Watson pass word'
speak_auth = ubinascii.b2a_base64(speak_username +':'+ speak_password)[:-1].decode("utf-8")
speak_headers={'Content-Type':'application/json','Authorization':'Basic %s' % speak_auth,'Accept':' */*'}

speak("message_ready.mp3")  #準備完了アナウンス
while True:
    fw = open('speech.wav', 'wb')   #録音用バッファファイルオープン
    dmy = fw.write(soundheader)     #録音用バッファファイルヘッダの書き込み
    print("ボタンを押してください")
    while btn.value() == 1: pass   #ボタンが押されるまで待ち
    vs.VS1003_Volume(240, 240)      #音声出力時スピーカボリューム
    speak("message_question.mp3")   #質問アナウンス
    vs.VS1003_Volume(50, 50)        #録音時にスピーカボリュームを絞る
    vs.VS1003_SetRecMode()          #録音モードセット

    vs.VS1003_Rec(fw, datalen)      #録音処理
    beep()                          #録音終了音出力
    fw.close()

    vs.VS1003_Volume(240, 240)      #音声出力時スピーカボリューム
    speak("wait.mp3")
 
    #録音した音声をSpeech to Textで音声認識
    r = req.post(recog_url, headers=recog_headers, file = 'speech.wav')
 
   #音声認識結果（json）をテキストに変換
    res =json.loads(r)
    text = ""
    for result in res['results']:
        for alternative in result['alternatives']:
            text += alternative['transcript']
        text = text.replace(" ","")
        print("Watson:%s\n" % text)
    if not("天気" in text):         #認識結果に天気の単語があるか調べる
        speak("message_no.mp3")      #単語が無ければそれをアナウンスして再入力
        continue
    id = 0
    name = ""
    for idx, na in weather_id.id_list:#認識結果に地域IDに該当する単語があるか調べる
        if na in text:
            if len(na) > len(name):
                name = na
                id = idx
    if id == 0:                        #該当が無ければそれをアナウンスして再入力
        speak("message_notfind.mp3")
        continue
    speak("message_search.mp3")

    #LiveDoorの天気予報サイトから指定地域の天気予報を取ってくる
    data = req.get(weather_url+id)

    #天気予報テキストから必要箇所を抜き出して編集
    dec = json.loads(data)
    text = dec["title"]+".\n"
    for n in range(len(dec["forecasts"])):  
        try:
            d = dec["forecasts"][n]["date"].split("-")
            text += "\n"
            text += d[1]+"月"+d[2]+"日.\n"
            text += dec["forecasts"][n]["telop"].replace("曇","曇り").replace("曇りり","曇り")+".\n"
            text += "最高気温"+dec["forecasts"][n]["temperature"]["max"]["celsius"].replace('-','氷点下')+"度.\n"
            text += "最低気温"+dec["forecasts"][n]["temperature"]["min"]["celsius"].replace('-','氷点下')+"度.\n"
        except:
            pass
    text += "天気概況.\n\n"+dec["description"]["text"]+".\n"
    print(text)

    #天気予報テキストをText to Speechで音声合成
    r = req.post(speak_url, headers=speak_headers, data='{"text":"'+text.replace("\n","")+'"}',stream = buf)

    data = dec = text = r =""
    gc.collect()
