'''
 *category   Micropython application
 *package    speak weather forecast library
 *author     Ichiro Shirasaka
 *copyright  2018 Ichiro Shirasaka All Rights Reserved
 *version    1.0'''
import time
from machine import Pin
import usocket

led = Pin(2,Pin.OUT)

# VS1003ドライバ
class Vs1003:

    def __init__(self, spi, xrst, xdcs, xcs, dreq):
        self.spi = spi
        self.xrst = xrst
        self.xdcs = xdcs
        self.xcs  = xcs
        self.dreq = dreq

    def VS1003_SCI_Enable(self):
        self.xcs .value(1)
        self.xdcs.value(1)
        self.xcs .value(0)

    def VS1003_SDI_Enable(self):
        self.xdcs.value(1)
        self.xcs .value(1)
        self.xdcs.value(0)

    def VS1003_Wait_dreq(self):
        while not self.dreq.value():
            #time.sleep_ms(5)
            led.value(1)
        led.value(0)
        
    def VS1003_SCI_Write(self, Address, wData):
        self.VS1003_SCI_Enable()
        
        self.VS1003_Wait_dreq()
        data = bytearray(4)
        data[0] = 0x02
        data[1] = Address
        data[2],data[3] = ((wData >> 8) & 0xFF), (wData & 0xFF)
        self.spi.write(data)
        #time.sleep_ms(5)
        self.xcs .value(1)

    def VS1003_SCI_Read(self, Address):
        wRecvData = 0x0000
        self.VS1003_SCI_Enable()

        self.VS1003_Wait_dreq()
        data = bytearray(2)
        data[0] = 0x03
        data[1] = Address
        self.spi.write(data)
        #time.sleep_ms(5)
        rdata = self.spi.read(3)[0:2]
        wRecvData = int.from_bytes(rdata,'big')
        self.xcs .value(1)
        return wRecvData

    def VS1003_Reset(self):
        self.xrst.value(0)
        self.xdcs.value(1)
        self.xcs .value(1)
        time.sleep_ms(100)
        self.xrst.value(1)
        time.sleep_ms(100)
        self.xrst.value(1)
        time.sleep_ms(100)
        self.VS1003_SCI_Write(0x00, 0x0804)
        time.sleep_ms(100)
        self.VS1003_SCI_Write(0x03, 0xa800)
        time.sleep_ms(100)
        
    def VS1003_SetRecMode(self):
        self.VS1003_SCI_Write(0x03, 0x9000)
        self.VS1003_SCI_Write(0x0c, 18)  #SCI AICTRL0
        self.VS1003_SCI_Write(0x0d, 0)   #SCI AICTRL1
        self.VS1003_SCI_Write(0x0e, 0)#SCI AICTRL2
        self.VS1003_SCI_Write(0x00, 0x1804)#reset new mode
        time.sleep_ms(100)

    def VS1003_Volume(self, iLeft, iRight):
        _left = 254 - iLeft
        _right = 254 - iRight
        attenuation = ((256 * _left) + _right)
        self.VS1003_SCI_Write(0x0B, attenuation)
        
    def VS1003_Rec(self, f, length):
        num = 0
        self.VS1003_Volume(100, 100)
        while num < length:
            ready = self.VS1003_SCI_Read(0x09)
            if ready >= 256:
                data = bytearray(2)
                data[0] = 0x03
                data[1] = 0x08
                self.VS1003_SCI_Enable()
                for n in range(0,256,2):
                    self.xcs .value(0)
                    self.VS1003_Wait_dreq()
                    self.spi.write(data)
                    f.write(self.spi.read(3)[0:2])
                    self.xcs .value(1)
                num += n

#HTTP リクエストユーテリティ
class Request:

    def __init__(self, spi, xdcs, dreq):
        self.spi = spi
        self.xdcs = xdcs
        self.dreq = dreq

    def streamout(self, sckt, stream):  #音声ストリーム出力（ネット→音声ボード）
        buf_len = 1024
        while buf_len == 1024:
            led.value(0)
            buf_len = sckt.readinto(stream)
            rcount = 0
            while rcount < 1024:
                if self.dreq.value() == 1:
                    self.xdcs.value(0)
                    led.value(1)
                    self.spi.write(stream[rcount:rcount+32])
                    rcount += 32
                    self.xdcs.value(1)
            #print("%d %d %d" % (quant, rcount, wcount))

    def request(self, method, url, data=None, json=None, headers={}, file=None, stream=None):
        try:
            proto, dummy, host, path = url.split("/", 3)
        except ValueError:
            proto, dummy, host = url.split("/", 2)
            path = ""
        if proto == "http:":
            port = 80
        elif proto == "https:":
            import ussl
            port = 443
        else:
            raise ValueError("Unsupported protocol: " + proto)

        if ":" in host:
            host, port = host.split(":", 1)
            port = int(port)

        ai = usocket.getaddrinfo(host, port)
        addr = ai[0][-1]

        s = usocket.socket()
        try:
            s.connect(addr)
            if proto == "https:":
                s = ussl.wrap_socket(s, server_hostname=host)
            s.write(b"%s /%s HTTP/1.0\r\n" % (method, path))
            if not "Host" in headers:
                s.write(b"Host: %s\r\n" % host)
            # Iterate over keys to avoid tuple alloc
            for k in headers:
                s.write(k)
                s.write(b": ")
                s.write(headers[k])
                s.write(b"\r\n")
                #print(k)
                #print(headers[k])
            if json is not None:
                assert data is None
                import ujson
                data = ujson.dumps(json)
            if data:
                s.write(b"Content-Length: %d\r\n" % len(data.encode("utf-8")))
                #print(b"Content-Length: %d\r\n" % len(data.encode("utf-8")))
            if file:
                fb = open(file, 'rb')
                data_len = fb.seek(1000000)
                s.write(b"Content-Length: %d\r\n" % data_len)
                fb.close()
            s.write(b"\r\n")
            if data:
                s.write(data)
                #print(data)
            if file:
                buf = bytearray(512)
                fb = open(file, 'rb')
                read_len = 0
                while read_len < data_len:
                    read_len += fb.readinto(buf)
                    s.write(buf)
                    #print(read_len)
                fb.close()
            l = s.readline()
            protover, status, msg = l.split(None, 2)
            status = int(status)
            print(protover, status, msg)
            while True:
                l = s.readline()
                if not l or l == b"\r\n":
                    break
                #print(l)
                if l.startswith(b"Transfer-Encoding:"):
                    if b"chunked" in l:
                        raise ValueError("Unsupported " + l)
                elif l.startswith(b"Location:") and not 200 <= status <= 299:
                    raise NotImplementedError("Redirects not yet supported")
        except OSError:
            s.close()
            raise
        if stream:
            self.streamout(s, stream)
        else:
            data = s.read()
        s.close()
        return data

    def head(self, url, **kw):
        return self.request("HEAD", url, **kw)

    def get(self, url, **kw):
        return self.request("GET", url, **kw)

    def post(self, url, **kw):
        return self.request("POST", url, **kw)

    def put(self, url, **kw):
        return self.request("PUT", url, **kw)

    def patch(self, url, **kw):
        return self.request("PATCH", url, **kw)

    def delete(self, url, **kw):
        return self.request("DELETE", url, **kw)